#!/usr/bin/perl
#********************************************************************************
# Tivoli Storage Manager (TSM)
# 
# name: tsmdiskperf
#
# desc: Simulates two typical TSM workload patterns for assessing
#       the suitability of disk storage for TSM.  The two workloads are
#       1) 256K sequential read+write simulating overlapped backup ingest
#          and identify duplicates
#       2) 16K random read/write simulating TSM database i/o
#
# usage:  perl tsmdiskperf.pl workload=stgpool|db [fslist=/fs1,/fs2,...]
#
# Notice: This program is provided as a tool intended for use by IBM Internal, 
#         IBM Business Partners, and IBM Customers. This program is provided as is, 
#         without support, and without warranty of any kind expressed or implied.
#
# (C) Copyright International Business Machines Corp. 2013, 2014
#********************************************************************************
BEGIN
{
 use Config;
 die "Threads not supported\n" unless $Config{'usethreads'};
}

use threads;

$versionString = "Program version 2.3";
$platform = getPlatform();
$SS = getpathdelimiter($platform);
if ($platform eq "WIN32")
{
  $ddName = "wdeedee.exe";
  $ioFlagR = "dio=3";
  $ioFlagW = "dio=3";
}
else
{
  $ddName = "dd";
  if($platform eq "AIX"){
  	$ioFlagR = "";
 	$ioFlagW = "";  	
  }else{
  	$ioFlagR = "iflag=direct";
 	$ioFlagW = "oflag=direct";
  }
}

srand(time);

if (@ARGV < 1 || @ARGV > 2)
{
  print "USAGE: tsmdiskperf.pl workload=stgpool|db [fslist=/fs1,/fs2,...]\n\n";
  exit (1);
}

my ($workload, $fslist, $fsCount, @fsArray);
$fslist = "";
$fsCount = 0;

if (@ARGV == 1)
{
  $workload = $ARGV[0];
  if ($workload !~ m/workload=(stgpool|db)/i)
  {
    print "USAGE: tsmdiskperf.pl workload=stgpool|db [fslist=/fs1,/fs2,...]\n\n";
    exit (1);
  }
  else
  {
    $workload = lc($1);
  }
}
elsif (@ARGV == 2)
{
  ($workload,$fslist) = @ARGV;

  if ($workload !~ m/workload=(stgpool|db)/i)
  {
    print "USAGE: tsmdiskperf.pl workload=stgpool|db [fslist=/fs1,/fs2,...]\n\n";
    exit (1);
  }
  else
  {
    $workload = lc($1);
  }

  if ($fslist !~ m/fslist=([\S|,]+)/i)
  {
    print "USAGE: tsmdiskperf.pl workload=stgpool|db [fslist=/fs1,/fs2,...]\n\n";
    exit (1);
  }
  else
  {
    $fslist = $1;
    # remove any trailing slashes from file system names
    $fslist =~ s/(\${SS},|\${SS}$)/,/g;

    @fsArray = split(',', $fslist);
    $fsCount = @fsArray;
  }
}


my ($blockSizeKB, $blockCount, $blockCountDB, $i, $j, $k, @semArray);
my ($dbThreadPerFS, $dbIOPerThread, $dbBatchIO, $totalDbIO);

# Variables
$filePrefix = "tsmdiskperf";       # prefix of filenames like tsmdiskperf0_1
$fileCount = 5;                    # number of files to iterate through per filesystem

# Some variable depend on the workload type
if ($workload eq "stgpool")
{
  $blockSizeKB="256";
  $fileSizeGB = 1;                   # size of each test file in GB
  if ($platform eq "WIN32")
  {
    $fsPrefix = "c:${SS}tsminst1${SS}TSMfile";   # prefix of filesystems to test against
  }
  else
  {
    $fsPrefix = "/tsminst1/TSMfile";   # prefix of filesystems to test against
  }
  if ($fsCount == 0)
  {
    $fsCount = 7;                    # number of file systems to include in the test
  }
}
elsif ($workload eq "db")
{
  $blockSizeKB="8";
  $fileSizeGB = 5;                   # size of each test file in GB
  if ($platform eq "WIN32")
  {
    $fsPrefix = "c:${SS}tsminst1${SS}TSMdbspace";# prefix of filesystems to test against
    $dbThreadPerFS = 2;                # number of threads per FS for db workload -- use even number
    $dbIOperThread = 1250;             # number of i/o's per thread
    $dbBatchIO = 64;                   # consecutive count for dd command
  }
  else
  {
    $fsPrefix = "/tsminst1/TSMdbspace";# prefix of filesystems to test against
    $dbThreadPerFS = 2;                # number of threads per FS for db workload -- use even number
    $dbIOperThread = 2500;             # number of i/o's per thread
    $dbBatchIO = 32;                   # consecutive count for dd command
  }
  if ($fsCount == 0)
  {
    $fsCount = 3;                    # number of file systems to include in the test
  }

  $totalDbIO = $dbIOperThread*$dbBatchIO;   # IO total per thread for output purposes
}
else
{
   die "ERROR: invalid workload specified\n";
}


# Calculate the number of blocks need to reach the file size
$blockCount = (1024 * 1024 * $fileSizeGB) / ($blockSizeKB);
$blockCountDB = (1024 * 1024 * $fileSizeGB) / (1024) ;


# Build a list of the file systems to test if not given as an argument
# Also cleanup residual files from any previous test
for ($i=0; $i<$fsCount; $i++)
{
  if ($i < 10)
  {
    push(@fsArray, ${fsPrefix}."0".${i}) if ($fslist eq ""); 
  }
  else
  {
    push(@fsArray, ${fsPrefix}.${i}) if ($fslist eq ""); 
  }
  for ($j = 1; $j <= $fileCount; $j++)
  {
    unlink ${fsArray[$i]}.$SS.$filePrefix.$i."_".$j;
    unlink ${fsArray[$i]}.$SS.$filePrefix."_1";
  }
}


# Ensure all of the specified test directories exist
foreach $fs (@fsArray)
{
  if (! -d $fs)
  {
    print "ERROR: The directory $fs does not exist\n";
    exit (1);
  }
}


# Now build a list of file systems that are actually backed by the directory list
# There may be fewer file systems than directories for some types of devices
# This list is used later as we pull from iostat output
@realFSList = qw();
if ($platform eq "WIN32")
{
  @mountOutput = `mountvol`;
}
else
{
  @mountOutput = `mount`;
}
$isGPFS = 0;
foreach $fs (@fsArray)
{
  # Gradually trim off directories until a file system is found
  $path="";
  if ($platform eq "WIN32")
  {
    @dirParts = split (m/\\/, $fs);
  }
  else
  {
    @dirParts = split ('/', $fs);
    $dirParts[0] = $SS;
  }
  foreach $dir (@dirParts)
  {
    if ($dir =~ m/\w\:$/)        # Include Windows drive letter with preceeding slash
    {
      $path = uc($dir).$SS;      
    }
    else
    {
      $path = $path.$dir.$SS; 
    }
    $path =~ s/${SS}${SS}/${SS}/g;  # remove any double slashes
    push (@pathList, $path);
  }

  for ($i=$#pathList; $i>=0; $i--)
  {
    $path = $pathList[$i];
    if ($platform ne "WIN32")
    {
      chop($path);
    }

    if ((grep {m/\s\Q$path\E\s/} @mountOutput) && ($path ne "/"))
    {
        $path =~ s/(\w\:\S*)\\/$1/;    # Trim off trailing slash for Windows
        if (! grep {m/\Q$path\E/} @realFSList)   # Only add unique file systems
        {
          push (@realFSList, $path);
          if ( (grep {m/\s\Q$path\E\s+type\s+gpfs\s/} @mountOutput) ||
               (grep {m/\s\Q$path\E\s+mmfs\s/} @mountOutput))  # Determine if FS is GPFS
          {
            $isGPFS++;
            $ioFlagR = "";  # Recommendation is to disable direct-IO for TSM with GPFS
            $ioFlagW = "";  	
          }
        }
        last;
    }
  }
}


# For the db workload, need to precreate a 2-d array of random locations.
# random number generation is slow, and bottlenecks IO if we leave this task
# in the thread loops

@randomList = qw(qw());
for ($i = 0; $i < ($dbThreadPerFS * $fsCount); $i++)
{
  for ($j = 0; $j < $dbIOperThread; $j++)
  {
    $random = int(rand($blockCount - ($dbBatchIO + 1)));
    $randomList[$i][$j] = $random;
  }
}


print "===================================================================\n";
print ": TSM disk performance test\t($versionString)\n";
print ":\n";
print ": Workload type:\t\t$workload\n";
print ": Number of filesystems:\t$fsCount\n";
if ($workload eq "stgpool")
{
  print ": Files to write per fs:\t$fileCount\n";
}
elsif ($workload eq "db")
{
  print ": Thread count per FS:\t\t$dbThreadPerFS\n";
  print ": I/Os per thread:\t\t$totalDbIO\n";
}
print ": File size:\t\t\t$fileSizeGB GB\n";
print ":\n";
print "===================================================================\n";

# For DB workload, prime each filesystem with a large file

if ($workload eq "db")
{
  print ":\n";
  print ": Creating files of $fileSizeGB GB to simulate TSM DB.\n";
  foreach $fs (@fsArray)
  {
    my $ddcmd = $ddName." if=/dev/zero of=".$fs.$SS.${filePrefix}."_1 bs=1024k count=$blockCountDB $ioFlagW 2>&1";

    print ": Issuing command $ddcmd\n";
    `$ddcmd`;
  }

}


# Sub routine for write thread for stgpool workload
sub writeThreadStgpool
{
  my $indx = shift(@_);
  my $self = threads->self();

  print ": Starting write thread ID: ", $self->tid, " on filesystem $fsArray[${indx}]\n";
 
  for (my $q = 1; $q <= $fileCount; $q++)
  {
    
    my $ddcmd = $ddName." if=/dev/zero of=".$fsArray[${indx}].$SS."${filePrefix}${indx}_${q} bs=${blockSizeKB}k count=$blockCount $ioFlagW 2>&1";

    # Write the file which will later be read
    #print "Issuing command $ddcmd\n";
    `$ddcmd`;

  }
}


# Sub routine for read thread for stgpool workload
sub readThreadStgpool
{
  my $indx = shift(@_);
  my $self = threads->self();

  print ": Starting read thread ID: ", $self->tid, " on filesystem $fsArray[${indx}]\n";


  # Read all except the last file
  for (my $q = 1; $q < $fileCount; $q++)
  {
    # Wait until the next file exists before reading the previous
    my $next = $q + 1;
    my $nextFile = $fsArray[${indx}].$SS.$filePrefix.${indx}."_".$next;
    while (! -e $nextFile)
    {
      sleep 1;
    }

    if (! -e "iostatout.txt")
    {
      # Create iostat file to allow the iostat collection to begin
      open (IOSTAT, ">>iostatout.txt") or die "ERROR opening file: iostatout.txt\n";
      close (IOSTAT);
    }

    my $ddcmd = $ddName." if=".$fsArray[${indx}].$SS.$filePrefix."${indx}_${q} of=/dev/null bs=${blockSizeKB}k $ioFlagR 2>&1";

    #print "Issuing command: $ddcmd\n";
    `$ddcmd`;
  }

}


# Sub routine for random write thread for DB workload
sub writeThreadDb
{
  my $pth = shift(@_);
  my $self = threads->self();
  my $seeknum = 0;
  my $tid = $self->tid();

  #print "Write thread ", $tid, " started \n";

  for (my $k = 1; $k <= $dbIOperThread; $k++)
  {
    $seeknum = $randomList[$tid - 1][$k - 1];

    my $ddcmd = $ddName." if=/dev/zero of=".${pth}.$SS.$filePrefix."_1 bs=${blockSizeKB}k count=${dbBatchIO} conv=notrunc seek=${seeknum} $ioFlagW 2>&1";

    #print "Issuing command $ddcmd\n";
    `$ddcmd`;

  }

}


# Sub routine for random read thread for DB workload
sub readThreadDb
{
  my $pth = shift(@_);
  my $self = threads->self();
  my $seeknum = 0;
  my $tid = $self->tid;

  #print "Read thread ", $tid, " started \n";

  # Create iostat file to allow the iostat collection to begin
  if (! -e "iostatout.txt")
  {
    open (IOSTAT, ">>iostatout.txt") or die "ERROR opening file: iostatout.txt\n";
    close (IOSTAT);
  }

  # Read one quarter fewers times as write
  for (my $k = 1; $k <= $dbIOperThread - ($dbIOperThread / 4); $k++)
  {
    $seeknum = $randomList[$tid - 1][$k - 1];

    my $ddcmd = $ddName." if=".${pth}.$SS.$filePrefix."_1 of=/dev/null bs=${blockSizeKB}k count=${dbBatchIO} skip=${seeknum} $ioFlagR 2>&1";

    #print "Issuing command $ddcmd\n";
    `$ddcmd`;

  }
}


# Begin I/O test here
print ":\n";
print ": Beginning I/O test.\n";
print ": The test can take upwards of ten minutes, please be patient ...\n";

# Record the starting time
$startTime = time();

# Get the correspondence between filesystems and devices from lsblk or mountvol output
@deviceArray = ();

if ($isGPFS > 0)  # Prepare to use the mmpmon command
{
  $commandReset = "commandReset.txt";
  $commandFile = "commandFile.txt";

  # Verify the mmpmon command is present
  $mmpmon = `which mmpmon 2>&1`;
  if ($mmpmon =~ m/no\s+mmpmon/)
  {
    $mmpmon = "/usr/lpp/mmfs/bin/mmpmon";
    if (! -f $mmpmon)
    {
      die "ERROR: Unable to locate the mmpmon command\n";
    }
  }
  chomp ($mmpmon);

  open (CMDFILE, ">$commandReset") or die "ERROR opening file: $commandReset\n";
  print CMDFILE "reset\n";
  close (CMDFILE);
  open (CMDFILE, ">$commandFile") or die "ERROR opening file: $commandFile\n";
  print CMDFILE "io_s\n";
  close (CMDFILE);

  @deviceArray = @realFSList;
}
else
{
  if ($platform eq "WIN32")  # For Windows, prepare list of counters to monitor
  {
    open (COUNTERS, ">wincounters.txt") or die "ERROR opening file: wincounters.txt\n";
    @counterList = ("Disk Reads/sec", "Disk Writes/sec", "Disk Transfers/sec",
                  "Disk Read Bytes/sec", "Disk Write Bytes/sec", "% Idle Time");

    foreach $fs (@realFSList)
    {
      push(@deviceArray, "\\LogicalDisk(".$fs.")\\*");
      foreach $counter (@counterList)
      {
        print COUNTERS "\\LogicalDisk(".$fs.")\\".$counter."\n";
      }
    } 

    close (COUNTERS);
  }
  elsif ( $platform eq "AIX" ) {
    @dfOut = `df`;
    @mountOut = `mount`;
    foreach $fs (@realFSList) {
      # Warn about any file systems that are not mount with dio option
      if (!grep {m/\Q$fs\E.*dio/} @mountOut)
      {
        print "WARNING: File system $fs is not mounted with dio option\n";
      }

      foreach $dfoutline (@dfOut) {
        if ( $dfoutline =~ m/^\/dev\/(\S+)\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+$fs\s*/ ){
          $lvName  = $1;
          @lvlsOut = `lslv -l $lvName`;
          foreach $lvlsoutline (@lvlsOut) {
            next if ($lvlsoutline =~ m/^PV\s+COPIES\s+IN BAND\s+DISTRIBUTION\s*/);
            if ($lvlsoutline =~ m/^(\S+)\s+\S+\s+\S+\s+\S+\s*/ ) {
              $pvName = $1;
              if ( iscontained( $pvName, \@deviceArray ) == 0 ) {
                push( @deviceArray, $pvName );
              }
            }
          }
        }
      }
    }
  }
  else
  {
    @lsblkOut = `lsblk -l -o NAME,MOUNTPOINT`;
    @mpathOut = `multipath -l`;

    foreach $lsblkoutline (@lsblkOut)
    {
      if ($lsblkoutline =~ m/\S*\s*\(*(dm-\d+|fio\w+|sd\w+|rbd\d+|md\d+|^\S\S\S\S\S\S\S\S\S\S\S+)\)*\s+(\S+)/)
      {
        $devnm = $1; 
        $fsmntpnt = $2;
  
        if (iscontained($fsmntpnt, \@realFSList) == 1)
        {
          if ($devnm !~ m/(dm-\d+|fio\w+|sd\w+|rbd\d+|md\d+)/)   # with rhel 7 need to get device short name from multipath output
          {
            foreach $mpathln (@mpathOut)
            {
              if ($mpathln =~ m/\Q$devnm\E\s+(\S+)\s+/)
              {
                $devnm = $1;
              }
            }
          }
          if (iscontained($devnm, \@deviceArray) == 0) 
          {
            push(@deviceArray, $devnm);
          }
        }
      }
    }
  }
}

# Cleanup any existing iostat output file
unlink ("iostatout.txt");

# Launch a separate process for running iostat to collect performance data
if ($runiostat = fork())
{
}
else
{
  # We wait for overlapped read and write activity to avoid contaminating the
  # iostat averages taken for read
  while (! -e "iostatout.txt")
  {
    sleep 1;
  }
  if ($isGPFS > 0)
  {
    `$mmpmon -p -i $commandReset`;   # reset performance counters
    unlink ("$commandReset");
    exec("$mmpmon -p -i $commandFile -d 3000 -r 0 > iostatout.txt 2>&1");
  }
  elsif ($platform eq "WIN32")
  {
    exec("typeperf -cf wincounters.txt -f csv -o iostatout.txt -si 7 -y > NUL");
    $perfProg = "typeperf";
  }
  elsif ($platform eq "AIX"){
  	exec("iostat -TdDl 7 > iostatout.txt");
  	$perfProg = "iostat";
  }
  else
  {
    exec("iostat -xtk 7 > iostatout.txt");
    $perfProg = "iostat";
  }
}


# Launch IO workloads in separate threads
my @threads;

if ($workload eq "stgpool")
{
  for ($i = 0; $i < $fsCount; $i++)
  {
    push(@threads, threads->create('writeThreadStgpool', $i));
    push(@threads, threads->create('readThreadStgpool', $i));
  }
}
elsif ($workload eq "db")
{
  my $threadcount = 1;
  foreach $fs (@fsArray)
  {
    for ($i=0; $i < $dbThreadPerFS - 1; $i++)
    {
      push(@threads, threads->create('writeThreadDb', $fs));
      push(@threads, threads->create('readThreadDb', $fs));
    }
  }
}

sleep 5;
while (my $thread = shift @threads)
{
  $thread->join();
}



if ($platform eq "WIN32")
{
  $qproccmd = "tasklist | findstr typeperf.exe";

  @qproccmdOut = `$qproccmd`;

  foreach $qprocOutln (@qproccmdOut)
  {
    if ($qprocOutln =~ m/typeperf.exe\s+(\d+)\s+\w+/)
    {
      $iostatprocid = $1;
    }
  }

  print ": All threads are finished.  Stopping typeperf process with id $iostatprocid\n";
  system("taskkill /pid $iostatprocid /f");
}
elsif ( $platform eq "AIX" ){
  $qproccmd = "ps -ef";

  @qproccmdOut = `$qproccmd`;

  foreach $qprocOutln (@qproccmdOut)
  {
    if ( ($isGPFS < 1) && ($qprocOutln =~ m/root\s+(\d+)\s+\d+\s+\d+\s+\S+\s+\S+\s+\S+\s+iostat -TdDl/) )
    {
      $iostatprocid = $1;
    }
    elsif ( ($isGPFS > 0) && ($qprocOutln =~ m/root\s+(\d+)\s+\d+\s+\d+\s+\S+\s+.*mmpmon\s+3000\s+/) )
    {
      $iostatprocid = $1;
    }
  }

  print ": All threads are finished.  Stopping iostat process with id $iostatprocid\n";
  system("kill -2 $iostatprocid");
}
else
{
  if ($isGPFS > 0)
  {
    $qproccmd = "ps -ef | grep mmpmon";
    $signal = "-2";
  }
  else
  {
    $qproccmd = "ps -ef | grep iostat";
    $signal = "-2";
  }

  @qproccmdOut = `$qproccmd`;

  foreach $qprocOutln (@qproccmdOut)
  {
    if ($qprocOutln =~ m/root\s+(\d+)\s+\d\d+\s+\d+\s+\S+\s+\S+\s+\S+\s+iostat\s+-xtk/)
    {
      $iostatprocid = $1;
    }
    elsif ($qprocOutln =~ m/root\s+(\d+)\s+\d\d+\s+\d+\s+\S+\s+\S+\s+\S+\s+.*mmpmon\s+3000\s+/)
    {
      $iostatprocid = $1;
    }
  }

  print ": All threads are finished.  Stopping iostat process with id $iostatprocid\n";
  system("kill $signal $iostatprocid");
}


# record the ending time, and calculate number of seconds
$endTime = time();
$elapsedSeconds = $endTime - $startTime;

# Cleanup test files
for ($i=0; $i<$fsCount; $i++)
{
  for ($j = 1; $j <= $fileCount; $j++)
  {
    unlink $fsArray[$i].$SS.$filePrefix.$i."_".$j;
    unlink $fsArray[$i].$SS.$filePrefix."_1";
  }
}
if ($isGPFS > 0)
{
  unlink ("$commandFile");
  unlink ("$runmmpmon");
}


print "===================================================================\n";
print ":  RESULTS:\n";


open(IOSTATH, "<iostatout.txt") or die "Unable to open iostatout.txt\n";
@iostatcontents = <IOSTATH>;
close IOSTATH;

# For Windows, reformatting typeperf output to match the Linux iostat output
if ($platform eq "WIN32")
{
  foreach $line (@iostatcontents)
  {
    next if ($line !~ m/\d\d\/\d\d\/\d\d\d\d/);
    @elements = split (',', $line);
    $item = shift(@elements)."\n";
    $item =~ s/\"//g;
    push (@newcontents, $item);     # Write the time stamp on its own line
    foreach $d (@deviceArray)
    {
      $newline = $d;
      for ($i = 0; $i <= $#counterList; $i++)
      {
        $item = shift(@elements);
        $item =~ s/\"//g;
        $newline = $newline."\t".$item;
      }
      push (@newcontents, $newline."\n");
    }
  }
  @iostatcontents = @newcontents;
  unlink ("wincounters.txt");
}


if (@deviceArray < 1)
{
  print ":\n";
  print ":  ERROR: Unable to map file systems from the fslist to device names.\n";
  print ":         Result analysis will be skipped.\n";
  exit (1);
}
else
{
  print ":  Devices reported on from $perfProg output:\n";
  foreach $d (@deviceArray)
  {
    print ":  $d\n";
  }
  print ":\n";
}

$laststanzadate = "";
$laststanzatime = "";
$stanzatotal = 0;

@stanzaInfoArray = ();

# Summarize the iostat or mmpmon output
if ($isGPFS > 0)
{
  $lastSec = 0;

  foreach $iostatline (@iostatcontents)
  {
    if($iostatline =~ m#^\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+\s+(\S+)\s+\S+\s+(\S+)\s+\S+\s+(\S+)\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+\s+(\S+)\s+\S+\s+#)
    {  
      $numSec = $1;    # _t_
      $micSec = $2;    # _tu_
      $bytesR = $3;    # _br_
      $bytesW = $4;    # _bw_
      $numReads = $5;  # _rdc_
      $numWrites = $6;  # _wc_

      if ($lastSec != 0)
      {
        $elapsed = ($numSec - $lastSec) + (($micSec - $lastMic) / 1000000);
        $curReads = $numReads - $lastReads;
        $curWrites = $numWrites - $lastWrites;
        $iopstotal = ($curReads + $curWrites) / $elapsed;
        $curBR = $bytesR - $lastBR;
        $curBW = $bytesW - $lastBW;
        $thrpRtotal = getCapByKB($curBR) / $elapsed;
        $thrpWtotal = getCapByKB($curBW) / $elapsed;
        $thrpCtotal = getCapByKB($curBR + $curBW) / $elapsed;

        my $stanzainfo = {};  # start new info hash for this iostat stanza
        $stanzainfo->{date} = "";
        $stanzainfo->{time} = $numSec; 
        $stanzainfo->{iopstotal} = $iopstotal;
        $stanzainfo->{thrptotal} = $thrpCtotal;
        $stanzainfo->{thrpRtotal} = $thrpRtotal;
        $stanzainfo->{thrpWtotal} = $thrpWtotal;
		
        push(@stanzaInfoArray, $stanzainfo);

      }

      $lastSec = $numSec;
      $lastMic = $micSec;
      $lastBR = $bytesR;
      $lastBW = $bytesW;
      $lastReads = $numReads;
      $lastWrites = $numWrites;
    }

  }#end for loop
}

elsif ($platform eq "AIX")
{
  # AIX iostat repeats headers mid-stanza, so throw away extras
  $lastTime = "";
  $header = "";
  @tmpOut = qw();
  foreach $iostatline (@iostatcontents)
  {
    if($iostatline =~ m#^Disks:\s+xfers\s+read\s+write\s+queue\s+time\s*#)
    {
      $header = $iostatline;
    }
    elsif ($iostatline =~ m#hdisk.*(\d\d:\d\d:\d\d)#)
    {
      if ($lastTime eq $1)
      {
        push (@tmpOut, $iostatline);
      }
      else
      {
        $lastTime = $1;
        push (@tmpOut, $header);
        push (@tmpOut, $iostatline);
      }
    }
  }
  @iostatcontents = @tmpOut;

  foreach $iostatline (@iostatcontents)
  {
    if($iostatline =~ m#^Disks:\s+xfers\s+read\s+write\s+queue\s+time\s*#)
    {    
      if ($laststanzatime ne "")
      {
        my $stanzainfo = {};  # start new info hash for this iostat stanza
	
        $stanzainfo->{date} = $laststanzadate;
        $stanzainfo->{time} = $laststanzatime; 
        $stanzainfo->{iopstotal} = $stanzaiopstotal;
        $stanzainfo->{thrptotal} = $stanzathrptotal;
        $stanzainfo->{thrpRtotal} = $stanzathrpRtotal;
        $stanzainfo->{thrpWtotal} = $stanzathrpWtotal;
		
        push(@stanzaInfoArray, $stanzainfo);
      }
      $laststanzadate = "";
      $laststanzatime = "";
      $stanzaiopstotal = 0;  # reset to 0
      $stanzathrptotal = 0;
      $stanzathrpRtotal = 0;
      $stanzathrpWtotal = 0;
    }
    elsif($iostatline =~ m#^(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)\s+(\S+)\s+(\S+)\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\d\d:\d\d:\d\d)#)
    {  
      #Disks:      xfers     read    write   queue   time
      $devc = $1;
      $riops = $4; #r/s->rps
      $wiops = $5; #w/s->wps
      $rthrp = getCapByKB($2); #rkB/s ->bread
      $wthrp = getCapByKB($3); #wkB/s ->bwrtn
	      
      $laststanzatime = $6;
      if (iscontained($devc, \@deviceArray) == 1)
      {
        $stanzaiopstotal += ($riops + $wiops);
        $stanzathrpRtotal += $rthrp;
        $stanzathrpWtotal += $wthrp;
        $stanzathrptotal += ($rthrp + $wthrp);
      }
    }	
  }#end for loop
}
else   # Linux86
{
  foreach $iostatline (@iostatcontents)
  {
    if ($iostatline =~ m#(\d\d/\d\d/\d\d\d\d)\s+(\d\d:\d\d:\d\d)#)  # get time stamp at start of new stanza
    {
      $currstanzadate = $1;
      $currstanzatime = $2;
	
      if ($laststanzatime ne "")
      {
        my $stanzainfo = {};  # start new info hash for this iostat stanza
	
        $stanzainfo->{date} = $laststanzadate;
        $stanzainfo->{time} = $laststanzatime; 
        $stanzainfo->{iopstotal} = $stanzaiopstotal;
        $stanzainfo->{thrptotal} = $stanzathrptotal;
        $stanzainfo->{thrpRtotal} = $stanzathrpRtotal;
        $stanzainfo->{thrpWtotal} = $stanzathrpWtotal;
	
        push(@stanzaInfoArray, $stanzainfo);
      }
	
      $laststanzadate = $currstanzadate;
      $laststanzatime = $currstanzatime;
      $stanzaiopstotal = 0;  # reset to 0
      $stanzathrptotal = 0;
      $stanzathrpRtotal = 0;
      $stanzathrpWtotal = 0;
    }
    elsif ($iostatline =~ m/(\S+)\s+(\d+.\d\d+)\s+(\d+.\d\d+)\s+(\d+.\d\d+)\s+(\d+.\d\d+)\s+(\d+.\d\d+)\s+(\d+.\d\d+)/)
    {
      if ($platform eq "WIN32")
      {
        $devc = $1;
        $riops = $2;
        $wiops = $3;
        $rthrp = $5/1024;
        $wthrp = $6/1024;
      }
      else
      {
        $devc = $1;
        $riops = $4;
        $wiops = $5;
        $rthrp = $6;
        $wthrp = $7;
      }
	
      if (iscontained($devc, \@deviceArray) == 1)
      {
        $stanzaiopstotal += ($riops + $wiops);
        $stanzathrpRtotal += $rthrp;
        $stanzathrpWtotal += $wthrp;
        $stanzathrptotal += ($rthrp + $wthrp);
      }
    }
  }  # end of for loop
}

# Get the last stanza, including cumulative throughput for averaging

my $stanzainfo = {};  # start new info hash for this iostat stanza

$stanzainfo->{date} = $laststanzadate;
$stanzainfo->{time} = $laststanzatime;
$stanzainfo->{iopstotal} = $stanzaiopstotal;
$stanzainfo->{thrpRtotal} = $stanzathrpRtotal;
$stanzainfo->{thrpWtotal} = $stanzathrpWtotal;
$stanzainfo->{thrptotal} = $stanzathrptotal;

push(@stanzaInfoArray, $stanzainfo);
 
$overalliopstotal = 0;
$overallthrpRtotal = 0;
$overallthrpWtotal = 0;
$stanzacount = 0;
$maxiops = 0;
$maxRthrp = 0;
$maxWthrp = 0;
$maxCmbThrp = 0;
$maxiopsstanza = 0;
$maxthrpRstanza = 0;
$maxthrpWstanza = 0;
$maxthrpCmbstanza = 0;

$numstanzas = @stanzaInfoArray;

for ($p = 0; $p < $numstanzas; $p++)  
{
  $overalliopstotal += $stanzaInfoArray[$p]->{iopstotal};
  $overallthrpRtotal += $stanzaInfoArray[$p]->{thrpRtotal};
  $overallthrpWtotal += $stanzaInfoArray[$p]->{thrpWtotal};

  if ($stanzaInfoArray[$p]->{iopstotal} > $maxiops)
  {
    $maxiops = $stanzaInfoArray[$p]->{iopstotal};
    $maxiopsstanza = $stanzacount;
  }

  if ($stanzaInfoArray[$p]->{thrpRtotal} > $maxRthrp)
  {
    $maxRthrp = $stanzaInfoArray[$p]->{thrpRtotal};
    $maxthrpRstanza = $stanzacount;
  }
  if ($stanzaInfoArray[$p]->{thrpWtotal} > $maxWthrp)
  {
    $maxWthrp = $stanzaInfoArray[$p]->{thrpWtotal};
    $maxthrpWstanza = $stanzacount;
  }
  if ($stanzaInfoArray[$p]->{thrptotal} > $maxCmbThrp)
  {
    $maxCmbThrp = $stanzaInfoArray[$p]->{thrptotal};
    $maxthrpCmbstanza = $stanzacount;
  }
  $stanzacount++;
}

$averageiops = $overalliopstotal / $stanzacount;
$averageRthrp = $overallthrpRtotal / $stanzacount;
$averageWthrp = $overallthrpWtotal / $stanzacount;
$averageCthrp = ($averageRthrp + $averageWthrp) / 1024;
$maxCthrp = $maxCmbThrp / 1024;

printf (":  Average R Throughput (KB/sec):\t%.2f\n",$averageRthrp);
printf (":  Average W Throughput (KB/sec):\t%.2f\n",$averageWthrp);
printf (":  Avg Combined Throughput (MB/sec):\t%.2f\n",$averageCthrp);
printf (":  Max Combined Throughput (MB/sec):\t%.2f\n",$maxCthrp);

$iopspeak = $stanzaInfoArray[$maxiopsstanza]->{iopstotal};
$iopspeakdate = $stanzaInfoArray[$maxiopsstanza]->{date};
$iopspeaktime = $stanzaInfoArray[$maxiopsstanza]->{time};

$thrpRpeak = $stanzaInfoArray[$maxthrpRstanza]->{thrpRtotal};
$thrpWpeak = $stanzaInfoArray[$maxthrpWstanza]->{thrpWtotal};
$thrpRpeakdate = $stanzaInfoArray[$maxthrpRstanza]->{date};
$thrpWpeakdate = $stanzaInfoArray[$maxthrpWstanza]->{date};
$thrpRpeaktime = $stanzaInfoArray[$maxthrpRstanza]->{time};
$thrpWpeaktime = $stanzaInfoArray[$maxthrpWstanza]->{time};
#$thrppeaktime = $stanzaInfoArray[$maxthrpstanza]->{time};

print ":\n";
printf (":  Average IOPS:\t\t\t%.2f\n",$averageiops);
printf (":  Peak IOPS:\t\t\t\t%.2f at %s %s\n",$iopspeak, $iopspeakdate, $iopspeaktime);
# We are no longer showing peak throughput.  Our method of extracting peak across multiple disks readings
# from an iostat interval is not reliable
#print ":  Peak Read Throughput (KB/sec):\t$thrpRpeak at $thrpRpeakdate $thrpRpeaktime\n";
#print ":  Peak Write Throughput (KB/sec):\t$thrpWpeak at $thrpWpeakdate $thrpWpeaktime\n";
print ":\n";
print ":  Total elapsed time (seconds):\t$elapsedSeconds\n";
print "===================================================================\n";

sub iscontained($$)
{
  local $d = shift(@_);
  local $dRef = shift(@_);

  local $found = 0;
  local $numds = @{$dRef};
  local $p = 0;

  while (($p < $numds) && ($found == 0))
  {
    if ("$d" eq "$dRef->[$p]")
    {
      $found = 1;
    }
    ++$p;
  }
  return $found;
}

############################################################
#      sub: getPlatform
#     desc: Returns the platform type.  The type returned
#           represent constants specifying the operating system. 
#           The type is determined based on the Perl built-in
#           ^O variable. 
#   params: None
#  returns: $string containing platform type constant
############################################################
sub getPlatform()
{
  $platfrm = $^O;      # $^O is built-in variable containing osname
  if ($platfrm =~ m#^aix#)                  { return "AIX" };
  if ($platfrm =~ m#^MSWin32#)              { return "WIN32" };
  
  if ($platfrm =~ m#^linux#)
  {
     my @uname = `uname -a`;
   
     foreach (@uname)
     {
       if  ($_ =~ m#x86_64#)   
       {
	 return "LINUX86";
       }
     }	
  }
  # We haven't found a match yet, so return UNKNOWN
  return "UNKNOWN";
}

sub getpathdelimiter
{
  $pltfrm = shift(@_);

  if ($platform eq "WIN32")
  {
    return "\\";
  }
  else
  {
    return "/";
  }
}

sub getCapByKB
{
	$cap = shift(@_);
	
	if($cap =~ m/([\d|\.]+)([KMGT])/){
		$capNew = int($1);
		$unit = $2;
		
		if($unit eq "K"){
			return $capNew;
		}elsif($unit eq "M"){
			return ($capNew * 1024);
		}elsif($unit eq "G"){
			return ($capNew * 1024 * 1024);
		}elsif($unit eq "T"){
			return ($capNew * 1024 * 1024 * 1024);
		}
	}
	return ($cap/1000);
}
